package livio.rssreader;
/*
Version 1.0, 16-08-2021, First release by Livio (javalc6@gmail.com)

IMPORTANT NOTICE, please read:

This software is licensed under the terms of the GNU GENERAL PUBLIC LICENSE,
please read the enclosed file license.txt or https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html

Note that this software is freeware and it is not designed, licensed or intended
for use in mission critical, life support and military purposes.

The use of this software is at the risk of the user.
*/
import java.util.ArrayList;


import androidx.appcompat.app.ActionBar;
import androidx.appcompat.widget.Toolbar;
import livio.rssreader.backend.FeedsDB;
import livio.rssreader.backend.IconArrayAdapter;
import livio.rssreader.backend.IconItem;
import androidx.appcompat.app.AppCompatActivity;
import livio.rssreader.backend.UserDB;
import tools.FormFactorUtils;

import android.content.SharedPreferences;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.annotation.SuppressLint;
import android.os.Bundle;
import android.widget.AdapterView;
import android.widget.ListView;
import android.widget.AdapterView.OnItemClickListener;
import android.content.Intent;

import com.google.android.material.dialog.MaterialAlertDialogBuilder;
import com.google.android.material.floatingactionbutton.FloatingActionButton;
import com.google.android.material.snackbar.Snackbar;

import static livio.rssreader.RSSReader.PREF_FEEDS_LANGUAGE;
import static livio.rssreader.backend.UserDB.CAT_SIZE;

public final class SelectCategory extends AppCompatActivity implements OnItemClickListener, AdapterView.OnItemLongClickListener {
	public final String tag = "SelectCategory";
	final static String ID_CATEGORY = "livio.rssreader.category";

    UserDB udb;
		
	@SuppressLint("NewApi")
	public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
		 Log.i(tag,"onCreate");
        setContentView(R.layout.categories);

        SharedPreferences prefs = androidx.preference.PreferenceManager.getDefaultSharedPreferences(this);

        udb = UserDB.getInstance(this, prefs);

        final Toolbar toolbar = findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);
		ActionBar actionBar = getSupportActionBar();
		if (actionBar != null) {
            actionBar.setDisplayHomeAsUpEnabled(!FormFactorUtils.isArc(this));
		}
        ArrayList<IconItem> catList = new ArrayList<>();
        String[] categories = getResources().getStringArray(R.array.categories_list);//localized names
		for (int k = 0; k < FeedsDB.categories.length; k++)
			catList.add(new IconItem(categories[k], FeedsDB.categories[k][1], false));

        for (int k = 0; k < udb.getUserCats().size(); k++)//user categories
            catList.add(new IconItem(udb.getUserCat(k)[0], null, false));

		
		// Create a customized ArrayAdapter
		IconArrayAdapter adapter = new IconArrayAdapter(
				this, R.layout.caticon_listitem, catList, FeedsDB.categories.length);
		
        ListView lv = findViewById(R.id.catlist);
		
		// Set the ListView adapter
		lv.setAdapter(adapter);
        lv.setOnItemClickListener(this);
        lv.setOnItemLongClickListener(this);
        final FloatingActionButton fab = findViewById(R.id.addfab);
        if (fab != null) {
            fab.setOnClickListener(v -> {
                getSupportFragmentManager().setFragmentResultListener("category_key", this, (requestKey, bundle) -> {
/*
        Log.d(tag, "category title:"+category[0]);
        Log.d(tag, "category description:"+category[1]);
        Log.d(tag, "category id:"+category[2]);
*/
// category[3] e category[4] sono riservate per usi futuri, per esempio per definire l'icona associata alla categoria utente
                    int position = udb.updateCategory(bundle.getStringArray("category"), this);
                    recreate();
                });
                NewCategoryDialog editNameDialog = NewCategoryDialog.newInstance(new String[CAT_SIZE]);//new category
                editNameDialog.show(getSupportFragmentManager(), "new_cat_dialog");
            });
        }
    }

	public void onItemClick(AdapterView<?> parent, View v, int position, long id) {

        Intent listActivity = new Intent(getBaseContext(), ListFeeds.class);
        Bundle b = new Bundle();
        if (position < FeedsDB.categories.length)
            b.putString("category", FeedsDB.categories[position][2]);
        else b.putString("category", udb.getUserCat(position - FeedsDB.categories.length)[2]);
        listActivity.putExtra(ID_CATEGORY, b);

        startActivityForResult(listActivity, REQUEST_CODE_PREFERENCES);
    }

    @Override
    public boolean onItemLongClick(AdapterView<?> adapterView, View view, int position, long id) {//edit a category
        if (position >= FeedsDB.categories.length) {//only user category can be edited
            String[] category = udb.getUserCat(position - FeedsDB.categories.length);
            getSupportFragmentManager().setFragmentResultListener("category_key", this, (requestKey, bundle) -> {
/*
        Log.d(tag, "category title:"+category[0]);
        Log.d(tag, "category description:"+category[1]);
        Log.d(tag, "category id:"+category[2]);
*/
// category[3] e category[4] sono riservate per usi futuri, per esempio per definire l'icona associata alla categoria utente
                udb.updateCategory(bundle.getStringArray("category"), this);
                recreate();
            });
            NewCategoryDialog editNameDialog = NewCategoryDialog.newInstance(category);
            editNameDialog.show(getSupportFragmentManager(), "edit_cat_dialog");
            return true;
        } else return false;
    }

	private static final int REQUEST_CODE_PREFERENCES = 1;  

	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent data) {
	    
// user exits from preference screen, process any changes	    
// close preference
	    if (requestCode == REQUEST_CODE_PREFERENCES) {
            setResult(resultCode);
	    	if (resultCode == RESULT_OK)  {
	   		 	finish();
	    	}
	    } else super.onActivityResult(requestCode, resultCode, data);
	}

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
//        Log.i(tag,"onCreateOptionsMenu");

        getMenuInflater().inflate(R.menu.select_cat_menu, menu);
        menu.findItem(R.id.menu_lang).setShowAsAction(MenuItem.SHOW_AS_ACTION_IF_ROOM);
        if (udb.getUserCats().size() > 0) {
            MenuItem del = menu.findItem(R.id.menu_delcat);
            del.setVisible(true);
            del.setShowAsAction(MenuItem.SHOW_AS_ACTION_IF_ROOM);
        }
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        // Handle item selection
        int itemId = item.getItemId();
        if (itemId == android.R.id.home) {
            setResult(RESULT_CANCELED);
            finish();
            return true;
        } else if (itemId == R.id.menu_lang) {
            selectLanguageDialog();
            return true;
        } else if (itemId == R.id.menu_delcat) { // delete a category
            ListView lv = findViewById(R.id.catlist);
            IconArrayAdapter iaa = (IconArrayAdapter) lv.getAdapter();
            int n_items = 0;
            int i = 0;
            while (i < iaa.getCount()) {
                IconItem aitem = iaa.getItem(i);
                if (aitem == null)
                    break;
                if (aitem.isChecked()) {
                    iaa.remove(aitem);
                    Log.d("******", "delcat=" + i);
                    udb.deleteCat(i);
                    n_items++;
                } else i++;
            }
            if (n_items == 0) {
                Snackbar.make(findViewById(android.R.id.content), getString(R.string.msg_nothing_to_delete), Snackbar.LENGTH_SHORT).show();
            } else {
                udb.synctoFile(this);
            }
            return true;
        }
        return super.onOptionsItemSelected(item);
    }

    private void selectLanguageDialog() {//custom dialog with all languages from resources
        final SharedPreferences prefs = androidx.preference.PreferenceManager.getDefaultSharedPreferences(this);
        final String[] languages = getResources().getStringArray(R.array.entries_list_preference);
        final String[] langcodes = getResources().getStringArray(R.array.entryvalues_list_preference);
        String pref_lang = prefs.getString(PREF_FEEDS_LANGUAGE, getString(R.string.default_feed_language_code));
        int checkeditem = -1;
        for (int i = 0; i < langcodes.length; i++) {
            if (langcodes[i].equals(pref_lang)) {
                checkeditem = i;
                break;
            }
        }
        new MaterialAlertDialogBuilder(this)
            .setTitle(R.string.dialog_title_list_preference)
            .setNegativeButton(android.R.string.cancel, (dialog, which) -> dialog.dismiss())
            .setSingleChoiceItems(languages, checkeditem, (dialog, which) -> {
                String langcode = langcodes[which];
//                        Log.d(tag, "langcode="+langcode);
                SharedPreferences.Editor editor = prefs.edit();
                editor.putString(PREF_FEEDS_LANGUAGE, langcode);
                editor.apply();
                dialog.dismiss();
            }).show();
    }

}
