package livio.rssreader.backend;
/*
Version 1.0, 16-08-2021, First release by Livio (javalc6@gmail.com)

IMPORTANT NOTICE, please read:

This software is licensed under the terms of the GNU GENERAL PUBLIC LICENSE,
please read the enclosed file license.txt or https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html

Note that this software is freeware and it is not designed, licensed or intended
for use in mission critical, life support and military purposes.

The use of this software is at the risk of the user.
*/
import java.io.IOException;
import java.util.List;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import livio.rssreader.R;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ArrayAdapter;
import android.widget.CheckBox;
import android.widget.ImageView;
import android.widget.TextView;

public final class IconArrayAdapter extends ArrayAdapter<IconItem> {

    private final LayoutInflater inflater;
    private static final String ASSETS_DIR = "images/";
    private final Context context;
    private final int resourceid;
    private final int native_categories;

    private final List<IconItem> iconItems;

    /** Holds child views for one row. */
    static class ItemViewHolder {
        final CheckBox checkBox;
        final TextView textView;
        final ImageView imageView;
        public ItemViewHolder(ImageView imageView, TextView textView, CheckBox checkBox) {
            this.checkBox = checkBox;
            this.textView = textView;
            this.imageView = imageView;
        }
    }

    public IconArrayAdapter(Context context, int textViewResourceId, List<IconItem> objects, int native_categories) {
        super(context, textViewResourceId, objects);
        inflater = LayoutInflater.from(context);
        this.context = context;
        this.iconItems = objects;
        this.resourceid = textViewResourceId;
        this.native_categories = native_categories;
    }

    public int getCount() {
        return iconItems.size();
    }

    public IconItem getItem(int index) {
        return iconItems.get(index);
    }

    @Override
    public View getView(int position, @Nullable View convertView, @NonNull ViewGroup parent) {
        // Item to display
        IconItem item = this.getItem(position);

        // The child views in each row.
        CheckBox checkBox;
        TextView textView;
        ImageView imageView;

        if (convertView == null) {// Create a new row view
            convertView = inflater.inflate(resourceid, null);

            // Find the child views.
            textView = convertView.findViewById(R.id.item_name);
            imageView = convertView.findViewById(R.id.item_icon);
            checkBox = convertView.findViewById(R.id.checkbox);

            // Optimization: Tag the row with it's child views, so we don't have to
            // call findViewById() later when we reuse the row.
            convertView.setTag(new ItemViewHolder(imageView, textView, checkBox));

            // If CheckBox is toggled, update the item it is tagged with.
            checkBox.setOnClickListener(v -> {
                CheckBox cb = (CheckBox) v;
                IconItem item1 = (IconItem) cb.getTag();
                item1.setChecked(cb.isChecked());
            });
        } else {// Reuse existing row view
            // Because we use a ViewHolder, we avoid having to call findViewById().
            ItemViewHolder viewHolder = (ItemViewHolder) convertView.getTag();
            checkBox = viewHolder.checkBox;
            textView = viewHolder.textView;
            imageView = viewHolder.imageView;
        }

        // Tag the CheckBox with the Item it is displaying, so that we can
        // access the item in onClick() when the CheckBox is toggled.
        checkBox.setTag(item);
        if (position >= native_categories)
            checkBox.setVisibility(View.VISIBLE);
        else checkBox.setVisibility(View.GONE);

        // Display item data
        checkBox.setChecked(item.isChecked());

        textView.setText(item.name);
        if (item.resourceId != null) {
            String imgFilePath = ASSETS_DIR + item.resourceId;
            try {
                Bitmap bitmap = BitmapFactory.decodeStream(this.context.getResources().getAssets()
                        .open(imgFilePath));
                imageView.setImageBitmap(bitmap);
            } catch (IOException e) {
                e.printStackTrace();
            }
        } else imageView.setVisibility(View.GONE);

        return convertView;
    }
}
