package livio.rssreader.backend;
/*
Version 1.0, 16-08-2021, First release by Livio (javalc6@gmail.com)

IMPORTANT NOTICE, please read:

This software is licensed under the terms of the GNU GENERAL PUBLIC LICENSE,
please read the enclosed file license.txt or https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html

Note that this software is freeware and it is not designed, licensed or intended
for use in mission critical, life support and military purposes.

The use of this software is at the risk of the user.
*/
import android.content.Context;
import android.text.Html;

import java.io.Serializable;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;

import androidx.annotation.NonNull;
import livio.rssreader.R;
import tools.DateParser;

public final class RSSItem implements Serializable, Comparable<RSSItem> {
	private static final long serialVersionUID = -7077485890855499754L;

	private String _title = "<untitled>";
	private String _description = null;
	private String _link = "";
	private String _category = "";
	private Date _pubdate = null;
	public final String tag = "RSSItem";	
	

    public int compareTo(RSSItem that) {
        if (that._pubdate == null)
            return -1;
        if (this._pubdate == null)
            return 1;
        return that._pubdate.compareTo(this._pubdate);
    }

	void setTitle(String title) {
		_title = title.trim();
	}
	void setDescription(String description) {
		_description = description.trim();
	}
	void setLink(String link) {
		_link = link;
	}
	void setCategory(String category) {
		_category = category;
	}

    void setPubDate(String pubdate)	{
        _pubdate = DateParser.parseDate(pubdate);
    }

    public String getTitle(boolean smart_title) {
        if (smart_title && (_description != null)) {//try to build smart title
            String text = _description.replaceAll("<figure.+?</figure>", "");//remove figure sections to avoid unwanted captions instead of smart titles
            text = Html.fromHtml(text).toString().replace("\uFFFC", "");//"\uFFFC" is OBJECT REPLACEMENT CHARACTER
            int end = text.indexOf('.');
            if ((end != -1) && end > 10 && end < 100) {//se troviamo il punto e la lunghezza compresa fra 10 e 100 caratteri-->restituisci smart title
                return text.substring(0, end);
            }
        }
        return _title;
    }

    public String getDescription() {
        if (_description == null)
            return _title; //fallback on title
		return _description;
	}

    public String getLink() {
		return _link;
	}

    String getCategory() {
		return _category;
	}


    private static final SimpleDateFormat dateformat24 = new SimpleDateFormat("yyyy-MM-dd HH:mm", Locale.US);
    private static final SimpleDateFormat dateformat12 = new SimpleDateFormat("yyyy-MM-dd h:mm a", Locale.US);//time

	public String getPubDate(Context context)	{
        boolean use24Hour = android.text.format.DateFormat.is24HourFormat(context);//time
        SimpleDateFormat dateformat = use24Hour ? dateformat24 : dateformat12;//time
		if (_pubdate == null)
			return dateformat.format(new Date());
		else return dateformat.format(_pubdate);
	}

    private final static String nice_time_simple = "%1$d %2$s %3$s";
    private final static String nice_time_hh_mm = "%1$d %2$s and %3$s %4$s %5$s";

    public String getNicePubDate(Context context)	{//user friendly format to display
        if (_pubdate == null)
            return "***";//date is missing
/*
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm");
        sdf.setTimeZone(TimeZone.getDefault());
*/
        long feed_age = (new Date().getTime() - _pubdate.getTime()) / 1000; //time difference in seconds
        if (feed_age < -60)
            return getPubDate(context);
        else if (feed_age < 60)
            return context.getString(R.string.now);
        else if (feed_age < 120) // < 2 minutes
            return String.format(nice_time_simple, 1, context.getString(R.string.minute), context.getString(R.string.ago));
        else if (feed_age < 3600) // < 60 minutes
            return String.format(nice_time_simple, feed_age / 60, context.getString(R.string.minutes), context.getString(R.string.ago));
        else if (feed_age < 43200) {// < 12 hours
            if ((feed_age % 3600) / 60 == 0) {// case: x hours and 0 minutes
                if (feed_age < 7200) // < 2 hours
                    return String.format(nice_time_simple, feed_age / 3600, context.getString(R.string.hour), context.getString(R.string.ago));
                else
                    return String.format(nice_time_simple, feed_age / 3600, context.getString(R.string.hours), context.getString(R.string.ago));
            } else {
                if (feed_age < 7200) // < 2 hours
                    return String.format(nice_time_hh_mm, feed_age / 3600, context.getString(R.string.hour), (feed_age % 3600) / 60, context.getString(R.string.minutes), context.getString(R.string.ago));
                else
                    return String.format(nice_time_hh_mm, feed_age / 3600, context.getString(R.string.hours), (feed_age % 3600) / 60, context.getString(R.string.minutes), context.getString(R.string.ago));
            }
        } else return getPubDate(context);
    }

    @NonNull
    public String toString() {
		if (_title == null) return "<untitled>";
/* limit how much text we display
		if (_title.length() > 42) {
			return _title.substring(0, 42) + "...";
		}
*/		
		return _title;
	}
}
